# -*- coding: utf-8 -*-
"""
    weppy_haml.cache
    ----------------

    Cache for Haml templates

    :copyright: (c) 2015 by Giovanni Barillari
    :license: BSD, see LICENSE for more details.
"""

from weppy._compat import PY2, hashlib_md5, to_bytes


def make_md5(value):
    if PY2:
        value = to_bytes(value)
    return hashlib_md5(value).hexdigest()[:8]


class HamlCache(object):
    data = {}
    hashes = {}

    def get(self, filename, source):
        hashed = make_md5(source)
        if self.hashes.get(filename) != hashed:
            return None
        return self.data.get(filename)

    def set(self, filename, source, compiled):
        self.data[filename] = compiled
        self.hashes[filename] = make_md5(source)



# -*- coding: utf-8 -*-
"""
    weppy_haml.ext
    --------------

    Provides the Haml extension for weppy

    :copyright: (c) 2015 by Giovanni Barillari
    :license: BSD, see LICENSE for more details.
"""

import os
import codecs
from weppy.extensions import Extension, TemplateExtension
from .hamlpy import Compiler
from .cache import HamlCache


def _read_source(filepath):
    return codecs.open(filepath, 'r', encoding='utf-8').read()


def _process_template(source):
    compiler = Compiler()
    return compiler.process_lines(source.splitlines())


def _store_compiled(filepath, code):
    outfile = codecs.open(filepath+".html", 'w', encoding='utf-8')
    outfile.write(code)


class Haml(Extension):
    namespace = 'Haml'
    default_config = dict(
        set_as_default=False,
        auto_reload=False
    )

    def _load_config(self):
        self.config.set_as_default = self.config.get(
            'set_as_default', self.default_config['set_as_default'])
        self.config.auto_reload = self.config.get(
            'auto_reload', self.default_config['auto_reload'])

    def on_load(self):
        self._load_config()
        self.env.debug = lambda: self.app.debug
        self.env.cache = HamlCache()
        self.app.add_template_extension(HamlTemplate)
        if self.config.set_as_default:
            self.app.template_default_extension = '.haml'
        for path, dirs, files in os.walk(self.app.template_path):
            for fname in files:
                if os.path.splitext(fname)[1] == ".haml":
                    filepath = os.path.join(path, fname)
                    source = _read_source(filepath)
                    code = _process_template(source)
                    _store_compiled(filepath, code)
                    if self.config.auto_reload:
                        relname = filepath.split(self.app.template_path+"/")[1]
                        self.env.cache.set(relname, source, code)


class HamlTemplate(TemplateExtension):
    namespace = 'Haml'
    file_extension = '.haml'

    def preload(self, path, name):
        if self.config.auto_reload or self.env.debug():
            filepath = os.path.join(path, name)
            source = _read_source(filepath)
            code = self.env.cache.get(name, source)
            if not code:
                code = _process_template(source)
                _store_compiled(filepath, code)
                self.env.cache.set(name, source, code)
        return path, name+".html"



# -*- coding: utf-8 -*-
"""
    weppy_haml.hamlpy
    -----------------

    Adapted from the original code (https://github.com/jessemiller/HamlPy)

    :copyright: (c) 2011 Jesse Miller
    :license: BSD, see LICENSE for more details.
"""

from .nodes import RootNode, HamlNode, create_node

VALID_EXTENSIONS = ['haml', 'hamlpy']


class Compiler:
    def __init__(self, options_dict=None):
        options_dict = options_dict or {}
        self.debug_tree = options_dict.pop('debug_tree', False)
        self.options_dict = options_dict

    def process(self, raw_text):
        split_text = raw_text.split('\n')
        return self.process_lines(split_text)

    def process_lines(self, haml_lines):
        root = RootNode(**self.options_dict)
        line_iter = iter(haml_lines)

        haml_node = None
        for line_number, line in enumerate(line_iter):
            node_lines = line

            if not root.parent_of(HamlNode(line)).inside_filter_node():
                if line.count('{') - line.count('}') == 1:
                    start_multiline = line_number # For exception handling

                    while line.count('{') - line.count('}') != -1:
                        try:
                            line = line_iter.next()
                        except StopIteration:
                            raise Exception('No closing brace found for multi-line HAML beginning at line %s' % (start_multiline+1))
                        node_lines += line

            # Blank lines
            if haml_node is not None and len(node_lines.strip()) == 0:
                haml_node.newlines += 1
            else:
                haml_node = create_node(node_lines)
                if haml_node:
                    root.add_node(haml_node)

        if self.options_dict and self.options_dict.get('debug_tree'):
            return root.debug_tree()
        else:
            return root.render()
